/*****************************************************************************/
/**
@file         conMeasurement.cpp
@copyright    (c) Rohde & Schwarz GmbH & Co. KG, Munich
@version      $Header: $
*
@author       benisch, 1GP1
@responsible  CB
*
@language     C++
*
@description  Example for the NRP VISA Passport
to demonstrate a simple PowerAverageMEasurement

@see          
*
@history
*  $Log: conMeasurement.cpp $
*
*******************************************************************************
******************************************************************************/

#include <stdio.h>
#include <conio.h>
#include "visa.h"
#include "windows.h"

#define BREAK_ON_ERROR(err) {ViStatus ec = err; if(ec != VI_SUCCESS) break; }

const ViUInt16 ErrQBit = 1<<2;
const ViUInt16 MAVBit  = 1<<4;
const ViUInt16 OperBit = 1<<7;


/* METHOD ********************************************************************/
/**
PrintError: print the Error Code in Hex Number to stdout
*
@param      ViStatus err : error code
*
@return     void
@todo           
******************************************************************************/
void PrintError(ViStatus err)
{
  if (err != VI_SUCCESS)
  {
    printf("Error: 0x%X\r\n", err);
  }
}

/* METHOD ********************************************************************/
/**
ReadSTB: Read the Statusbyte and print the according messages to stdout
*
@param      ViSession session :     in:  the VISA RSNRP session
@param      ViUInt16 *statusByte :  out: pointer to the Status Byte to be read
*
@return     ViStatus: returning the Error code (VI_SUCCESS if there was no error)
@todo           
******************************************************************************/
ViStatus ReadSTB(ViSession session, ViUInt16 *statusByte)
{
  ViStatus rc = VI_SUCCESS;
  do 
  {
    rc = viReadSTB(session, statusByte);
    BREAK_ON_ERROR(rc);

    if (*statusByte & ErrQBit)
      printf("STB: Error queue not empty\r\n");
    if (*statusByte & MAVBit)
      printf("STB: Message Available\r\n");
    if (*statusByte & OperBit)
      printf("STB: Sensor not IDLE\r\n");
    else
      printf("STB: Sensor in IDLE state\r\n");

  } while(0);

  printf("\r\n");
  return rc;

}

/* METHOD ********************************************************************/
/**
GetSensorState: queries the status operation register to find out the trigger
state of the sensor
*
@precondition  
@postcondition   
@side_effects  
*
@param      ViSession session : in:  the VISA RSNRP session
@param      long *State :       out: pointer to the Condition variable
*
@return     ViStatus: Errorcode (VI_SUCCESS if there was no error)
@todo           
******************************************************************************/
ViStatus GetSensorState(ViSession session, long *State)
{
  ViStatus rc = VI_SUCCESS;
  do 
  {
    unsigned char condCmd[] = "stat:oper:cond?\n";
    long cmdSize = sizeof(condCmd);
    unsigned long retCnt = 0;
    ViStatus status = viWrite(session, condCmd, cmdSize, &retCnt);      
    BREAK_ON_ERROR(status);

    const long READBUFSIZ = 256;
    unsigned char readBuff[READBUFSIZ];
    memset(readBuff, 0, sizeof(readBuff));
    retCnt = 0;
    status = viRead(session, readBuff, READBUFSIZ, &retCnt);
    BREAK_ON_ERROR(status);
    char* xx = 0;
    *State = strtol((const char*)readBuff, &xx, 0);
    printf("Sensor State = 0x%X\r\n\n", *State);
  } while(0);

  return rc;
}

/* METHOD ********************************************************************/
/**
ReadDeviceList: read all connected NRP Sensors from the USB Ports
*
@precondition  
@postcondition   
@side_effects  
*
@param      ViSession session : in:  the VISA RSNRP session
*
@return     void
@todo           
******************************************************************************/
void ReadDeviceList(ViSession mainSession)
{
  ViStatus status = VI_SUCCESS;

  char searchString[] = "RSNRP::?*";
  ViFindList findList = VI_NULL;
  ViUInt32 entryCount = 0;
  const long BUFFLEN = 256;
  char instrList[BUFFLEN];
  do 
  {
    memset(instrList, 0, BUFFLEN);
    status = viFindRsrc(mainSession, searchString, &findList, &entryCount, instrList);
    BREAK_ON_ERROR(status);

    printf("| %s\r\n", instrList);

    while(status == VI_SUCCESS)
    {
      status = viFindNext(findList,instrList);
      BREAK_ON_ERROR(status);
      printf("| %s\r\n", instrList);
    };
  } while(false);
}

/* METHOD ********************************************************************/
/**
OpenfirstNrp: Opens the first NRP Sensor that is found on the USB
*
@precondition  
@postcondition   
@side_effects  
*
@param      ViSession mainSession : ResourceManager Session opened with viOpenDefaultRM
@param      ViSession* pSession :  oput: pointer to the new opened NRP-Z session
*
@return     ViStatus: errorcode, VI_SUCCESSFUL if no error occurred
@todo           
******************************************************************************/
ViStatus OpenfirstNrp(ViSession mainSession, ViSession* pSession)
{
  char searchString[] = "RSNRP::?*";
  ViFindList findList = VI_NULL;
  ViUInt32 entryCount = 0;
  ViUInt32 timeout = 5000;
  const long BUFFLEN = 256;
  char firstDevice[BUFFLEN];
  memset(firstDevice, 0, BUFFLEN);
  ViStatus status = VI_SUCCESS;
  do 
  {
    status = viFindRsrc(mainSession, searchString, &findList, &entryCount, firstDevice);
    BREAK_ON_ERROR(status);
    status = viOpen(mainSession, firstDevice, VI_NULL, timeout, pSession);
    BREAK_ON_ERROR (status);
  } while(false);
  return status;
}



/* METHOD ********************************************************************/
/**
main
*
@precondition  
@postcondition   
@side_effects  
*
@return     int
@todo           
******************************************************************************/
int main(int argc, char* argv[])
{
  printf ("RSNRP Visa Plugin Example Measurement\r\n");
  printf ("========================================\r\n");

  ViObject  rsnrpObj = 0;
  ViSession mainSession = 0;
  ViSession nrpzSession = 0;
  ViStatus status = VI_SUCCESS;
  ViUInt32 timeout = 20000;

  do
  {
    status = viOpenDefaultRM(&mainSession);
    BREAK_ON_ERROR (status);

    status = OpenfirstNrp (mainSession, &nrpzSession);
    BREAK_ON_ERROR (status);

    status = viSetAttribute(nrpzSession, VI_ATTR_TMO_VALUE, timeout);
    BREAK_ON_ERROR (status);
    printf("timeout auf 20 gesetzt....\r\n");

    //Setup State Changed Event
    ViEventType eventType = VI_EVENT_SERVICE_REQ;
    status = viEnableEvent(nrpzSession, eventType,  VI_QUEUE, VI_NULL);
    BREAK_ON_ERROR(status);


    //=========================================================================
    //  Setup a Measurement:
    //
    unsigned long retCnt = 0;
    unsigned long cmdSize = 0;

    printf ("doing a measurement....\n"); 
    unsigned char rstCmd[]  = "*rst\n";
    unsigned char funcCmd[]  = "sens:func \"xtime:power\"\n";
    unsigned char avgOffCmd[]  = "sens:aver:stat on; sens:aver:count:auto off; sens:aver:count 128\n";
    unsigned char trigSourCmd[] = "sens:aux minmax;trig:sour imm\n";
    unsigned char startMeasCmd[] = "init:imm\n";
    cmdSize = sizeof(rstCmd);

    // *rst
    status = viWrite(nrpzSession,rstCmd, cmdSize, &retCnt);      
    BREAK_ON_ERROR(status);

    //Scope Mode
    cmdSize = sizeof(funcCmd);
    status = viWrite(nrpzSession, funcCmd, cmdSize, &retCnt);      
    BREAK_ON_ERROR(status);

    //average
    cmdSize = sizeof(avgOffCmd);
    status = viWrite(nrpzSession, avgOffCmd, cmdSize, &retCnt);      
    BREAK_ON_ERROR(status);

    //trigger sour
    cmdSize = sizeof(trigSourCmd);
    status = viWrite(nrpzSession, trigSourCmd, cmdSize, &retCnt);      
    BREAK_ON_ERROR(status);

    //init:imm
    cmdSize = sizeof(startMeasCmd);
    status = viWrite(nrpzSession, startMeasCmd, cmdSize, &retCnt);      
    BREAK_ON_ERROR(status);
    printf("init:imm\r\n");      


    long sensorState = -1;
    ViEvent eventData = 0;
    ViUInt16 statusByte = 0;

    ReadSTB(nrpzSession, &statusByte);


    printf("Wait until MAV...\r\n");
    statusByte = 0;
    while (! (statusByte & MAVBit))
    {
      status = ReadSTB(nrpzSession, &statusByte);
      BREAK_ON_ERROR(status);
      Sleep(250);
    }

    BREAK_ON_ERROR(status);
    ReadSTB(nrpzSession, &statusByte);

    // !!! Ist dieser Aufruf drin, kann kein Ergebnis abgeholt werden
    //GetSensorState(nrpzSession, &sensorState);
    // !!! Ist dieser Aufruf drin, kann kein Ergebnis abgeholt werden

    ReadSTB(nrpzSession, &statusByte);

    //fetch?
    const long READBUFSIZ = 10240;
    unsigned char readBuff[READBUFSIZ];
    unsigned char fetchCmd[] = "FETCH?\n";
    cmdSize = sizeof(fetchCmd);
    status = viWrite(nrpzSession, fetchCmd, cmdSize, &retCnt);      
    BREAK_ON_ERROR(status);

    memset(readBuff, 0, sizeof(readBuff));
    retCnt = 0;
    status = viRead(nrpzSession, readBuff, READBUFSIZ, &retCnt);
    BREAK_ON_ERROR(status);
    printf("\nResult: %s\r\n\n", readBuff);

    ReadSTB(nrpzSession, &statusByte);
    GetSensorState(nrpzSession, &sensorState);


    //syst:err?
    printf("SYStem:ERRor?");
    unsigned char sysErrCmd[] = "syst:error?\n";
    cmdSize = sizeof(sysErrCmd);
    status = viWrite(nrpzSession, sysErrCmd, cmdSize, &retCnt);      
    BREAK_ON_ERROR(status);

    memset(readBuff, 0, sizeof(readBuff));
    retCnt = 0;
    status = viRead(nrpzSession, readBuff, READBUFSIZ, &retCnt);
    BREAK_ON_ERROR(status);
    printf("%s\r\n\n", readBuff);

  }while(0);

  viClose(nrpzSession);

  PrintError(status)  ;
  printf("\r\n");
  printf ("Hit 'Enter' to Exit\r\n");
  _getch();


  return 0;
}
